#ifndef __EX94317_H
#define __EX94317_H


typedef 	char				i8;
typedef 	unsigned char			u8;
typedef 	short				i16;
typedef 	unsigned short			u16;
typedef 	long				i32;
typedef 	unsigned long			u32;
typedef 	float				f32;
typedef 	double				f64;

struct ThreadParameter
{
       u8   CardID;
       bool enable_irq;
       bool ThreadProc;
       bool ThreadSet;
       u32  ThreadTimeOut;
};

//******************************************************
#define		_CARD_ID_MAX			15
#define		_NO_CARD			0xFF
#define		EX94317_PORT_MIN		0
#define		EX94317_PORT_MAX		5
#define		EX94317_IN_POINT_MAX  		7
#define		EX94317_OUT_POINT_MAX		7
#define		EX94317_INDEX_MIN		0
#define		EX94317_INDEX_MAX		2
#define		EX94317_DEBOUNCE_TIME_MAX	3

//#define		SUCCESS				1
#define		FAIL				0


////////////// Error Code ///////////////////////
#define		DRV_NO_ERROR			0

/************ Driver Error ***************/
#define		DRV_READ_DATA_ERROR	 	1
#define		DRV_INIT_ERROR			2

/************ Device Error ***************/
#define		DEVICE_RW_ERROR			100
#define		DRV_NO_CARD			101
#define		DRV_DUPLICATE_ID		102
#define		DRV_NOT_INSTALL			103

/************ User Parameter Error ********/
#define		ID_ERROR			300
#define		PORT_ERROR			301
#define		IN_POINT_ERROR			302
#define		OUT_POINT_ERROR			303
#define		SOURCE_ERROR			305
#define		DEBOUNCE_MODE_ERROR		306

#define		EVENT_ERROR			310
/************ Time Counter Error ********/
#define		INDEX_ERROR			406

/************ Offset address *************/
#define		PORT0				0x00    //Input inport0 address
#define		IO00				0
#define		IO01				1
#define		IO02				2
#define		IO03				3
#define		IO04				4
#define		IO05				5
#define		IO06				6
#define		IO07				7

#define		PORT1				0x01    //Input inport1 address
#define		IO10				0
#define		IO11				1
#define		IO12				2
#define		IO13				3
#define		IO14				4
#define		IO15				5
#define		IO16				6
#define		IO17				7

#define		PORT2				0x02    //Input inport2 address
#define		IO20				0
#define		IO21				1
#define		IO22				2
#define		IO23				3
#define		IO24				4
#define		IO25				5
#define		IO26				6
#define		IO27				7

#define		PORT3				0x03    //Input inport3 address
#define		IO30				0
#define		IO31				1
#define		IO32				2
#define		IO33				3
#define		IO34				4
#define		IO35				5
#define		IO36				6
#define		IO37				7

#define		PORT4				0x04    //Input inport4 address
#define		IO40				0
#define		IO41				1
#define		IO42				2
#define		IO43				3
#define		IO44				4
#define		IO45				5
#define		IO46				6
#define		IO47				7

#define		PORT5				0x05    //Input inport5 address
#define		IO50				0
#define		IO51				1
#define		IO52				2
#define		IO53				3
#define		IO54				4
#define		IO55				5
#define		IO56				6
#define		IO57				7

#define		NO_DEBOUNCE_TIME		0	//SET DEBOUNCE_TIME
#define		DEBOUNCE_TIME_100HZ		1
#define		DEBOUNCE_TIME_200HZ		2
#define		DEBOUNCE_TIME_1KHZ		3

#define		TC_CONTROL			0	//TC_function
#define		PRELOAD				1	
#define		COUNTER				2

#define		STOP				0	//tc_control
#define		RUN				1

#define		IO				0	//Interrupt source
#define		TC				1
#define		PCI_ENABLE		2
//-------------------------------------------------------------


#ifdef __cplusplus
	extern "C"{
#endif

//mark by linux #define EX94317_Status __declspec(dllexport) APIENTRY  //Standard calling convention
#define EX94317_Status	extern i32

/********************* DLL Function ********************************/
EX94317_Status EX94317_initial	(int *fd);
EX94317_Status EX94317_close	(int fd);
EX94317_Status EX94317_info	(u8 CardID, u16 *IO_address, u16 *TC_address);


EX94317_Status EX94317_config_set(u8 CardID,u8 configuration);
EX94317_Status EX94317_config_read(u8 CardID,u8 *configuration);

EX94317_Status EX94317_port_set(u8 CardID,u8 port, u8 data);
EX94317_Status EX94317_port_read(u8 CardID,u8 port, u8 *data);
EX94317_Status EX94317_point_set(u8 CardID,u8 port,u8 point,u8 state);
EX94317_Status EX94317_point_read(u8 CardID,u8 port,u8 point,u8 *state);

EX94317_Status EX94317_debounce_time_set(u8 CardID,u8 port,u8 debounce_time);
EX94317_Status EX94317_debounce_time_read(u8 CardID,u8 port,u8 *debounce_time);

//----------------------------TC-------------------------------------------------------
EX94317_Status EX94317_timer_set (u8 CardID, u32 time_constant);
EX94317_Status EX94317_timer_start(u8 CardID);
EX94317_Status EX94317_timer_stop(u8 CardID) ;
EX94317_Status EX94317_TC_set(u8 CardID,u8 index,u32 data);
EX94317_Status EX94317_TC_read(u8 CardID,u8 index,u32 *data);

//--------------------Interrupt--------------------------------------------------------
EX94317_Status EX94317_IRQ_polarity_set(u8 CardID,u8 polarity);
EX94317_Status EX94317_IRQ_polarity_read(u8 CardID,u8 *polarity);

EX94317_Status EX94317_IRQ_mask_set (u8 CardID, u8 source, u8 mask);
EX94317_Status EX94317_IRQ_mask_read (u8 CardID, u8 source, u8 *mask);

EX94317_Status EX94317_IRQ_process_link(u8 CardID, void (*callbackAddr)(u8 CardID));
EX94317_Status EX94317_IRQ_enable	(u8 CardID);
EX94317_Status EX94317_IRQ_disable	(u8 CardID);
EX94317_Status EX94317_IRQ_status_read(u8 CardID, u8 source, u8 *Event_Status);


#ifdef __cplusplus
}
#endif


#endif  // belongs to #ifndef __EX94317_H#ifndef __cplusepluse

