
/*
This driver is modified from drv94317.c in Linux kernel 2.6.23
*/

#include <linux/version.h>
#include <linux/kernel.h>   /* printk() */
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/string.h>
#include <linux/fs.h>       /* everything... */
#include <linux/errno.h>    /* error codes */
#include <linux/init.h>
#include <linux/sched.h>      /* interrupt */
#include <linux/interrupt.h>  /* interrupt for request_irq(), free_irq() */
#include <linux/moduleparam.h>
#include <asm/io.h>  //inb,outb ...
#include <asm/uaccess.h>
#include <linux/cdev.h>       /* struct cdev */
#include <linux/kdev_t.h>     /* MKDEV(int major, int minor) */
#include <linux/spinlock.h>

#include "drv94317.h"

//#define UCLINUX
//#define DEBUG

MODULE_DESCRIPTION("EX94317");
MODULE_AUTHOR("TOPCCC");
MODULE_LICENSE("Proprietary");

int count=1;
int EX94317_found=0;                          //record of device opens
int EX94317_count=0;                          //record of founded 94317 cards
int drv94317_minor=EXDRV_MINOR;
int drv94317_major=EXDRV_MAJOR;

irq_resource irqtemp[16];

spinlock_t my_lock=SPIN_LOCK_UNLOCKED;

unsigned long Enable_Card[16];

static struct cdev *my_dev;

struct fasync_struct *EX94317_async_queue;

struct pci_dev *EX94317_dev[16];

pci_info pci_tbl[]=
{
  {
    .vendor_id = 0x1573,
    .device_id = 0x0100,
    .subvendor_id = 0x6936,
    .subdevice_id = 0x9431,
    .address_offset0 = 0,
    .address_offset1 = 1,
  },
  {0,},
};


//****************Devices Interrupt ISR process********************//
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,19)
static irqreturn_t  EX94317_interrupt(int irq, void *dev_id)
#else
static irqreturn_t  EX94317_interrupt(int irq, void *dev_id,struct pt_regs *regs)
#endif
{
  //spinlock_t mr_lock=SPIN_LOCK_UNLOCKED;

  irq_resource *check_CardID = dev_id;

#ifdef DEBUG
  printk("#EX94317 : (ISR) check CardID = %lx \n",check_CardID->CardID);
#endif
  irqtemp[check_CardID->CardID].pci_enable=((inb((irqtemp[check_CardID->CardID].base_address)+_PCI_IRQ_ENABLE))&0x01);
  irqtemp[check_CardID->CardID].irq_data=((inb((irqtemp[check_CardID->CardID].base_address)+_IRQ_STATUS))&0x0ff);
  irqtemp[check_CardID->CardID].irq_mask=(inb((irqtemp[check_CardID->CardID].base_address)+_IRQ_MASK))&0x0ff;
  irqtemp[check_CardID->CardID].irq_tcdata=((inw((irqtemp[check_CardID->CardID].tc_address)+_TC_IRQ_STATUS))&0x01);
  irqtemp[check_CardID->CardID].irq_tcmask=((inw((irqtemp[check_CardID->CardID].tc_address)+_TC_IRQ_MASK))&0x01);

  if (irqtemp[check_CardID->CardID].pci_enable && ((irqtemp[check_CardID->CardID].irq_data && irqtemp[check_CardID->CardID].irq_mask) || (irqtemp[check_CardID->CardID].irq_tcdata && irqtemp[check_CardID->CardID].irq_tcmask)))
    {
//       spin_lock(&mr_lock);
      outb(inb((irqtemp[check_CardID->CardID].base_address)+_IRQ_STATUS),(irqtemp[check_CardID->CardID].base_address)+_IRQ_STATUS);		//reset irq Lattice
      outw(inw((irqtemp[check_CardID->CardID].tc_address)+_TC_IRQ_STATUS),(irqtemp[check_CardID->CardID].tc_address)+_TC_IRQ_STATUS);		//reset tc irq Lattice

      irqtemp[check_CardID->CardID].irq_flag=1;//set flag

      kill_fasync( &EX94317_async_queue, SIGUSR1, POLL_IN );//call .dll
//      spin_unlock(&mr_lock);//spin unlock
      return IRQ_HANDLED;
    }

  return IRQ_NONE;

}

static int EX94317_fasync(int fd, struct file *filp, int mode )
{
  return	fasync_helper(fd, filp, mode, &EX94317_async_queue);
}


//****************Devices Find Card********************//

static int find_94317(void)
{
  struct pci_dev *dev_t=NULL;
#ifdef i386
  unsigned long CardID=0;
  unsigned long temp_base_address;    //2007.5.18
  unsigned long temp_tc_address;      //2007.5.18
#else
  unsigned int  CardID=0;
  unsigned int  temp_base_address;    //2007.5.18
  unsigned int  temp_tc_address;      //2007.5.18
#endif
  unsigned short i;

#ifdef UCLINUX
  int temp_base_address_ioremap;
  int temp_tc_address_ioremap;
#endif

  for (i=0;i<EX_MAXDEV;i++)
    {
      EX94317_dev[i]=NULL;
    }

  for (EX94317_found=0;EX94317_found<EX_MAXDEV;)
    {
      dev_t=pci_get_device(pci_tbl[0].vendor_id,pci_tbl[0].device_id ,dev_t );
      if (!dev_t)
        {
          break;
        }
      else
        {
          if ((pci_tbl[0].subvendor_id==dev_t->subsystem_vendor)&&(pci_tbl[0].subdevice_id==dev_t->subsystem_device))
            {
#ifdef DEBUG
              printk(KERN_WARNING"#EX94317 : subvendor=%x \n", dev_t->subsystem_vendor);
              printk(KERN_WARNING"#EX94317 : subsystem=%x \n", dev_t->subsystem_device);
#endif
              temp_base_address=pci_resource_start(dev_t,pci_tbl[0].address_offset0);
              temp_tc_address=pci_resource_start(dev_t,pci_tbl[0].address_offset1);
#ifdef UCLINUX
              temp_base_address_ioremap=(int)ioremap(temp_base_address,0x100);//remap virtual address to physical address
              temp_tc_address_ioremap=(int)ioremap(temp_tc_address,0x100);
              CardID=inb((temp_base_address_ioremap)+_CARD_ID);
              irqtemp[CardID].base_address=temp_base_address_ioremap;
              irqtemp[CardID].tc_address=temp_tc_address_ioremap;
#else

              CardID=inb((temp_base_address)+_CARD_ID);
              irqtemp[CardID].base_address=temp_base_address;
              irqtemp[CardID].tc_address=temp_tc_address;
#endif
              irqtemp[CardID].CardID = CardID;
              Enable_Card[EX94317_found]=CardID;
              EX94317_dev[CardID]=dev_t;
#ifdef DEBUG
              printk(KERN_WARNING "#EX94317 : CardID =%lx \n",CardID);
              printk(KERN_WARNING "#EX94317 : base_address=%lx \n", irqtemp[CardID].base_address);
              printk(KERN_WARNING "#EX94317 : tc_address=%lx \n", irqtemp[CardID].tc_address);
#endif
              EX94317_found++;
            }
        }

    }
#ifdef DEBUG
  printk(KERN_WARNING"#EX94317 : found= %x \n",EX94317_found);
#endif
  return EX94317_found;
}



//****************Devices I/O control********************//

static int EX94317_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
  int result;
  u8 irq_data;
#ifdef i386
  unsigned long  int_flag;
#else
  unsigned int   int_flag;
#endif

  card_resource *pdev=(card_resource*)arg;   //arg->card_resouce, save in *dev
#ifdef DEBUG
  printk(KERN_WARNING "#EX94317 : in ioctl\n");
#endif
  switch (cmd)
    {
    case EXPCI_FIND:
      pdev->in_iodata.b[0]=EX94317_found;
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : find=%x \n",EX94317_found);
#endif
      return SUCCESS;
//*******************************//
    case EXPCI_RESOURCE:

      pdev->base_address=irqtemp[Enable_Card[pdev->in_iodata.b[0]-1]].base_address;
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : DIO address= %lx\n",pdev->base_address);
#endif
      pdev->tc_address=irqtemp[Enable_Card[pdev->in_iodata.b[0]-1]].tc_address;
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : TC address= %lx\n",pdev->tc_address);
#endif
      if (SUCCESS==pci_enable_device(EX94317_dev[Enable_Card[pdev->in_iodata.b[0]-1]]))
        {
          return SUCCESS;
        }
      else
        return -ENODEV;
//*******************************//

    case EXPCI_HOOK_IRQ_SR:
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : hook CardID =%lx \n",pdev->CardID);
#endif
      if (EX94317_dev[pdev->CardID]!=NULL)
        {
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,19)
          result=request_irq(EX94317_dev[pdev->CardID]->irq,EX94317_interrupt,IRQF_SHARED,"EX94317",&irqtemp[pdev->CardID]);   //request irq fc8
#else
          result=request_irq(EX94317_dev[pdev->CardID]->irq,EX94317_interrupt,SA_SHIRQ,"EX94317",&irqtemp[pdev->CardID]);   //request irq  fc4
#endif
          if (result!=0)
            {
#ifdef DEBUG
              printk(KERN_WARNING "#EX94317 : result = %i\n",result);
              printk(KERN_WARNING "#EX94317 : can't get assigned irq \n");
#endif
            }
          else
            {
#ifdef DEBUG
              printk(KERN_WARNING "#EX94317 : get assigned irq = %i\n",EX94317_dev[pdev->CardID]->irq);
#endif

              outb(inb((irqtemp[pdev->CardID].base_address)+_IRQ_STATUS),(irqtemp[pdev->CardID].base_address)+_IRQ_STATUS);		//reset irq
              outw(inw((irqtemp[pdev->CardID].tc_address)+_TC_IRQ_STATUS),(irqtemp[pdev->CardID].tc_address)+_TC_IRQ_STATUS);		//reset tc irq Lattice
            }
          return SUCCESS;
        }
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : can't enable IRQ \n");
#endif
      return -ENODEV;

//*******************************//

    case EXPCI_UNHOOK_IRQ_SR:
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : unhook CardID =%lx \n",pdev->CardID);
#endif
      if (EX94317_dev[pdev->CardID]!=NULL)
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : base_address = %lx\n",irqtemp[pdev->CardID].base_address);
          printk(KERN_WARNING "#EX94317 : tc_address = %lx\n",irqtemp[pdev->CardID].tc_address);
          printk(KERN_WARNING "#EX94317 : free irq %x \n",EX94317_dev[pdev->CardID]->irq);
#endif
          free_irq(EX94317_dev[pdev->CardID]->irq,&irqtemp[pdev->CardID]);

          return SUCCESS;
        }
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : can't free irq \n");
#endif
      return -ENODEV;
//*******************************//

    case EXPCI_IRQ_STATUS_READ:

#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : IRQ_STATUS_READ CardID =%lx \n",pdev->CardID);
#endif
      switch (pdev->source)
        {
        case 0:
          irq_data=inb((irqtemp[pdev->CardID].base_address)+_IRQ_STATUS);
          outb(irq_data,(irqtemp[pdev->CardID].base_address)+_IRQ_STATUS);
          //printk(KERN_WARNING "#EX94317 : Dio irq_data =%x \n",irq_data);
          //printk(KERN_WARNING "#EX94317 : Dio isr irq_data =%x \n",irqtemp[pdev->CardID].irq_data);
          pdev->irq_data=(~irqtemp[pdev->CardID].irq_mask) & irq_data;
          //printk(KERN_WARNING "#EX94317 : Dio irq_data0 =%x \n",pdev->irq_data);
          pdev->irq_data=  pdev->irq_data | irqtemp[pdev->CardID].irq_data;
          irqtemp[pdev->CardID].irq_data=0;
          //printk(KERN_WARNING "#EX94317 : Dio irq_data1 =%x \n",pdev->irq_data);
          break;
        case 1:
          irq_data=inw((irqtemp[pdev->CardID].tc_address)+_TC_IRQ_STATUS);
          outw(irq_data,(irqtemp[pdev->CardID].tc_address)+_TC_IRQ_STATUS);
          //printk(KERN_WARNING "#EX94317 : tc irq_data =%x \n",irq_data);
          //printk(KERN_WARNING "#EX94317 : tc isr irq_data =%x \n",irqtemp[pdev->CardID].irq_tcdata);
          pdev->irq_tcdata=(~irqtemp[pdev->CardID].irq_tcmask) & irq_data;
          //printk(KERN_WARNING "#EX94317 : tc irq_data0 =%x \n",pdev->irq_tcdata);
          pdev->irq_tcdata=  pdev->irq_tcdata | irqtemp[pdev->CardID].irq_tcdata;
          irqtemp[pdev->CardID].irq_tcdata=0;
          //printk(KERN_WARNING "#EX94317 : aio irq_data1 =%x \n",pdev->irq_tcdata);
          break;
        }
      return SUCCESS;

//*******************************//
    case EXPCI_IRQ_FLAG:

      pdev->irq_flag=irqtemp[pdev->CardID].irq_flag;

#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : pdev->CardID =%lx \n",pdev->CardID);
      printk(KERN_WARNING "#EX94317 : pdev->irq_flag =%x \n",pdev->irq_flag);
#endif
      return SUCCESS;
//*******************************//
    case EXPCI_IRQ_CLEARFLAG:
      irqtemp[pdev->CardID].irq_flag=0;
      return SUCCESS;

//*******************************//
    case EXPCI_RBYTE:
      if (pdev->base_address!=0)
        {
          pdev->in_iodata.b[0]=inb((pdev->base_address)+(pdev->in_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RBYTE address= %lx \n",(pdev->base_address)+(pdev->in_offset));
          printk(KERN_WARNING "#EX94317 : RBYTE out_iodata.b= %x \n",pdev->in_iodata.b[0]);
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RBYTE bad address= %lx \n",(pdev->base_address)+(pdev->in_offset));
#endif
          return -INVALID_ADDRESS;
        }
//*******************************//
      /*        case EXPCI_RBYTEREG:
              if(pdev->reg_address!=0)
      	{
      	    pdev->in_iodata.b[0]=inb((pdev->reg_address)+(pdev->in_offset));
      #ifdef DEBUG
                  printk(KERN_WARNING "#EX94317 :RBYTEREG address= %lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                  return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
                printk(KERN_WARNING "#EX94317 :RBYTEREG bad address= %lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                 return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_WBYTE:

      if (pdev->base_address!=0)
        {
          outb(pdev->out_iodata.b[0],(pdev->base_address)+(pdev->out_offset));

#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WBYTE address= %lx \n",(pdev->base_address)+(pdev->out_offset));
          printk(KERN_WARNING "#EX94317 : WBYTE out_iodata.b= %x \n",pdev->out_iodata.b[0]);
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WBYTE bad address= %lx \n",(pdev->base_address)+(pdev->out_offset));
#endif
          return -INVALID_ADDRESS;
        }
//*******************************//
      /*	case EXPCI_WBYTEREG:
             	if(pdev->reg_address!=0)
      	{
      	    outb(pdev->out_iodata.b[0],(pdev->reg_address)+(pdev->out_offset));
      #ifdef DEBUG
                  printk(KERN_WARNING "#EX94317 :WBYTEREG address= %lx \n",(pdev->reg_address)+(pdev->out_offset));
      #endif
                  return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
               printk(KERN_WARNING "#EX94317 :WBYTEREG bad address= %lx \n",(pdev->reg_address)+(pdev->out_offset));
      #endif
                 return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_RWORD:
      if (pdev->base_address!=0)
        {
          pdev->in_iodata.w[0]=inw((pdev->base_address)+(pdev->in_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RWORD address= %lx \n",(pdev->base_address)+(pdev->in_offset));
          printk(KERN_WARNING "#EX94317 : RWORD data= %x \n",pdev->in_iodata.w[0]);
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RWORD bad address= %lx \n",(pdev->base_address)+(pdev->in_offset));
#endif
          return -INVALID_ADDRESS;
        }

//*******************************//
      /*     case EXPCI_RWORDREG:
       	if(pdev->reg_address!=0)
      	{
      	   pdev->in_iodata.w[0]=inw((pdev->reg_address)+(pdev->in_offset));
      #ifdef DEBUG
                 printk(KERN_WARNING "#EX94317 :RWORDREG address= %lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                 return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
                 printk(KERN_WARNING "#EX94317 :RWORDREG bad address= %lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                 return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_WWORD:
      if (pdev->base_address!=0)
        {
          outw(pdev->out_iodata.w[0],(pdev->base_address)+(pdev->out_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WWORD address= %lx \n",(pdev->base_address)+(pdev->out_offset));
          printk(KERN_WARNING "#EX94317 : WWORD data= %x \n",pdev->out_iodata.w[0]);
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WWORD bad address= %lx \n",(pdev->base_address)+(pdev->out_offset));
#endif
          return -INVALID_ADDRESS;
        }

//*******************************//
      /*      case EXPCI_WWORDREG:
      	if(pdev->reg_address!=0)
      	{
      	    outw(pdev->out_iodata.w[0],(pdev->reg_address)+(pdev->out_offset));
      #ifdef DEBUG
                  printk(KERN_WARNING "#EX94317 : WWORDREG address =%lx \n",(pdev->reg_address)+(pdev->out_offset));
      #endif
                  return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
               printk(KERN_WARNING "#EX94317 :WWORDREG bad address =%lx \n",(pdev->reg_address)+(pdev->out_offset));
      #endif
               return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_RLONG:
      if (pdev->base_address!=0)
        {
          pdev->in_iodata.l=inl((pdev->base_address)+(pdev->in_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RLONG address =%lx \n",(pdev->base_address)+(pdev->in_offset));
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RLONG bad address =%lx \n",(pdev->base_address)+(pdev->in_offset));
#endif
          return -INVALID_ADDRESS;
        }
//*******************************//
      /*
             case EXPCI_RLONGREG:
      	if(pdev->reg_address!=0)
      	{
      	    pdev->in_iodata.l=inl((pdev->reg_address)+(pdev->in_offset));
      #ifdef DEBUG
                  printk(KERN_WARNING "#EX94317 :RLONGREG address =%lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                  return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
                printk(KERN_WARNING "#EX94317 :RLONGREG bad address =%lx \n",(pdev->reg_address)+(pdev->in_offset));
      #endif
                return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_WLONG:
      if (pdev->base_address!=0)
        {
          outl(pdev->out_iodata.l,(pdev->base_address)+(pdev->out_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WLONG address =%lx \n",(pdev->base_address)+(pdev->out_offset));
#endif
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WLONG bad address =%lx \n",(pdev->base_address)+(pdev->out_offset));
#endif
          return -INVALID_ADDRESS;
        }
//*******************************//
      /*      case EXPCI_WLONGREG:

      	if(pdev->reg_address!=0)
      	{
      	   outl(pdev->out_iodata.l,(pdev->reg_address)+(pdev->out_offset));
      #ifdef DEBUG
                 printk(KERN_WARNING "#EX94317 :WLONGREG address =%lx \n",(pdev->reg_address)+(pdev->out_offset));
      #endif
                 return SUCCESS;
              }
              else
      	{
      #ifdef DEBUG
      #endif
               return -INVALID_ADDRESS;
              }
      */
//*******************************//
    case EXPCI_RINDEX:
      if (pdev->base_address!=0)
        {
          spin_lock_irqsave(&my_lock,int_flag);
          //local_irq_disable();
          outw(pdev->in_offset,pdev->base_address);// write index
          pdev->in_iodata.w[0] = inw((pdev->base_address) +(pdev->in_offset));
          //local_irq_enable();  //only minimum time of disable irq
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RINDEX pdev->base_address =%lx \n",pdev->base_address);
#endif
          spin_unlock_irqrestore(&my_lock,int_flag);
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : RINDEX bad pdev->base_address =%lx \n",pdev->base_address);
#endif
          return -INVALID_ADDRESS;
        }

//*******************************//
    case EXPCI_WINDEX:
      if (pdev->base_address!=0)
        {
          spin_lock_irqsave(&my_lock,int_flag);
          //local_irq_disable();
          outw(pdev->out_offset,pdev->base_address);// write index
          outw(pdev->out_iodata.w[0],(pdev->base_address)+(pdev->out_offset));
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WINDEX pdev->base_address =%lx \n",pdev->base_address);
#endif
          spin_unlock_irqrestore(&my_lock,int_flag);
          return SUCCESS;
        }
      else
        {
#ifdef DEBUG
          printk(KERN_WARNING "#EX94317 : WINDEX bad pdev->base_address =%lx \n",pdev->base_address);
#endif
          return -INVALID_ADDRESS;
        }
//*******************************//
    default:
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : No correct command ");
#endif
      return -INVALID_CMD;
    }
}


//****************Devices Open and Release********************//

static int EX94317_open(struct inode *inode, struct file *filp)
{
  EX94317_count++;
#ifdef DEBUG
  printk(KERN_WARNING"#EX94317 : open count=%x\n",EX94317_count);
#endif
  return SUCCESS;
}
//*******************************//
static int EX94317_release(struct inode *inode, struct file *filp)
{
  EX94317_count--;
#ifdef DEBUG
  printk(KERN_WARNING"#EX94317 : I'M IN release STATUS \n");
#endif
  return SUCCESS;
}



//****************Devices module ********************//
static int __init EX94317_init(void)
{
  int result,err,i;

  dev_t devno;

#ifdef DEBUG
  printk(KERN_WARNING"#EX94317 : I'm in init module \n");
#endif

  EX94317_found=find_94317();

  if (EX94317_found>0)
    {
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : found EX94317 = %d\n",EX94317_found);
#endif

      for (i=0;i<EX_MAXDEV;i++)
        {
          irqtemp[i].irq_flag=0;
        }

      result=alloc_chrdev_region(&devno,0,count,"EX94317");
      if (result <0)
        {
          printk(KERN_WARNING "#EX94317 : unable to get major %d\n", drv94317_major);
          return result;
        }

      drv94317_major=MAJOR(devno);
#ifdef DEBUG
      printk(KERN_WARNING "#EX94317 : Load driver EX94317 success  %d\n",drv94317_major);
#endif

      my_dev = cdev_alloc();
      my_dev->owner = THIS_MODULE;
      my_dev->ops = &ex_fops;
      err = cdev_add(my_dev,devno,count);

      if (err < 0)
        {
          printk(KERN_WARNING "#EX94317 : Error %d  can't find EX94317 deivce \n",err);
          return -INITIAL_FAIL;
        }
      else
        {
          printk(KERN_WARNING "#EX94317 : Adding EX94317 deivce success\n");
          return SUCCESS;
        }
    }
  else
    return -INITIAL_FAIL;
}

//*******************************//

static void EX94317_exit(void)
{
  dev_t devno =MKDEV(drv94317_major,drv94317_minor);

  cdev_del(my_dev);
  unregister_chrdev_region(devno,count);
  printk(KERN_WARNING "#EX94317 : I'm out of module \n");
}

module_init(EX94317_init);
module_exit(EX94317_exit);
